
setwd("SET_FILE_LOCATION")

library(dplyr)
library(ggplot2)
library(ncdf4)
library(tidyr)
library(purrr)
library(stringr)
library(scico)
library(hetGP)
library(RobustGaSP)
library(readr)
library(plotly)

theme_set(theme_bw())

source("_extendedLHC.R")

# ------------------------------------------------------------------------------
# -------------------------- Design wave 2 emulator ----------------------------
# ------------------------------------------------------------------------------

nd_info_raw <- read_csv("data/wave1_results.csv")

nd_scaled <- nd_info_raw %>%
  mutate(
    sed = (sed - min(nd_info_raw$sed)) / 
    (max(nd_info_raw$sed) - min(nd_info_raw$sed)),
    rs_tune = (rs_tune - min(nd_info_raw$rs_tune)) / 
    (max(nd_info_raw$rs_tune) - min(nd_info_raw$rs_tune)),
    scaling = (scaling - 1) / 3,
    nd_inv = nd_inv/(10^12),
    flux = flux/(10^9),
    residence = residence/(10^3)
  )

ggplot(nd_scaled) +
  geom_point(aes(x = rs_tune, y = sed, colour = nd_inv), size = 2) +
  scale_colour_scico(
    direction = 1, palette = "batlow", limits = c(0,18)
  )

nd_scaled_2d <- nd_scaled %>% filter(exp != "xpdav")

nd_emulator <- rgasp(
  design = nd_scaled %>% 
    dplyr::select(sed, rs_tune, scaling) %>% 
    as.matrix(),
  response = nd_scaled %>% 
    dplyr::select(nd_inv) %>% 
    as.matrix(),
  trend = cbind(
    matrix(rep(1, nrow(nd_scaled))),
    nd_scaled %>% dplyr::select(sed, rs_tune, scaling) %>% as.matrix()
  ),
  range.par = c(1.076963, 0.5386766, 1.5)
)

residence_emulator  <- rgasp(
  design = nd_scaled %>% 
    dplyr::select(sed, rs_tune, scaling) %>% 
    as.matrix(),
  response = nd_scaled %>% 
    dplyr::select(residence) %>% 
    as.matrix(),
  trend = cbind(
    matrix(rep(1, nrow(nd_scaled))),
    nd_scaled %>% dplyr::select(sed, rs_tune, scaling) %>% as.matrix()
  ),
  range.par = c(0.8429957, 0.3059769, 1)
)

n <- 20

pred_matrix <- expand.grid(
  seq(0, 1, length = n),
  seq(0, 1, length = n),
  seq(0, 1, length = n)
) %>% as.matrix()

nd_prediction <- predict(
  nd_emulator,
  pred_matrix,
  cbind( matrix(rep(1, n^3)), pred_matrix )
)

residence_prediction <- predict(
  residence_emulator,
  pred_matrix,
  cbind( matrix(rep(1, n^3)), pred_matrix )
)

plot_points <- expand.grid(
  sed = seq(0, 1, length = n), 
  rs_tune = seq(0, 1, length = n),
  scaling = seq(0, 1, length = n)
) %>%
  as_tibble() %>%
  mutate(
    nd_mean = nd_prediction$mean,
    nd_sd = nd_prediction$sd,
    nd_lower = nd_mean - 2 * nd_sd,
    nd_upper = nd_mean + 2 * nd_sd,
    res_mean = residence_prediction$mean,
    res_sd = residence_prediction$sd,
    res_lower = res_mean - 2 * res_sd,
    res_upper = res_mean + 2 * res_sd
  ) %>%
  filter(
    nd_upper > 3.5, nd_lower < 4.5,
    res_upper > 0.35, res_lower < 0.95
  )

  expand.grid(
    sed = seq(0, 1, length = n), 
    rs_tune = seq(0, 1, length = n),
    scaling = seq(0, 1, length = n)
  ) %>%
    as_tibble() %>%
    mutate(
      nd_mean = nd_prediction$mean,
      nd_sd = nd_prediction$sd,
      nd_lower = nd_mean - 2 * nd_sd,
      nd_upper = nd_mean + 2 * nd_sd,
      res_mean = residence_prediction$mean,
      res_sd = residence_prediction$sd,
      res_lower = res_mean - 2 * res_sd,
      res_upper = res_mean + 2 * res_sd
    )%>%
  filter(
    scaling == 0, nd_upper > 3.5, nd_lower < 4.5, 
    res_upper > 0.35, res_lower < 0.95
  ) %>%
  ggplot() +
    geom_tile(
      aes(
        x = rs_tune, y = sed, width = 1/(n-1), 
        height = 1/(n-1), fill = res_mean
      )
    ) +
    scale_x_continuous("rs", expand = c(0, 0), limits = c(0,1)) +
    scale_y_continuous("source", expand = c(0, 0), limits = c(0,1))

fig <- plot_ly(
  as.data.frame(plot_points), 
  x = ~rs_tune, 
  y = ~sed, 
  z = ~scaling,
  marker = list(color = ~nd_mean, colorscale = c('#683531', '#FFE1A1'), showscale = TRUE)
)

fig <- fig %>% add_markers()

fig

# ------------------------------------------------------------------------------
# -------------------------- Select Design Locations ---------------------------
# ------------------------------------------------------------------------------

TestDesign <- sim.anneal(
  D_candidates = as.matrix(plot_points %>% dplyr::select(sed, rs_tune, scaling)), 
  RestDesign = as.matrix(nd_scaled %>% dplyr::select(sed, rs_tune, scaling)),
  n=15, w=2, t0=1000, FAC_t=0.7, Imax = 500, p=50, range.space=c(-1,1)
)

final_design <- as.matrix(plot_points %>% dplyr::select(sed, rs_tune, scaling))[TestDesign, ] %>%
  as_tibble()

points_all <- bind_rows(
  nd_scaled %>% dplyr::select(sed, rs_tune, scaling),
  final_design
)

fig <- plot_ly(
  as.data.frame(points_all), 
  x = ~rs_tune, 
  y = ~sed, 
  z = ~scaling,
  marker = list(color = ~scaling, colorscale = c('#683531', '#FFE1A1'), showscale = TRUE)
)

fig <- fig %>% add_markers()

fig

pairs(final_design)

final_design %>%
  mutate(
    sed = sed * (max(nd_info_raw$sed) - min(nd_info_raw$sed)) + 
      min(nd_info_raw$sed),
    rs_tune = rs_tune * (max(nd_info_raw$rs_tune) - min(nd_info_raw$rs_tune)) + 
      min(nd_info_raw$rs_tune),
    scaling = scaling * 3 + 1
  ) %>%
  write.csv("data/wave2_design.csv")
